@@ -32,21 +32,19 @@
 
 #define GRN_CTX_INITIALIZER(enc) \
   { GRN_SUCCESS, 0, enc, 0, GRN_LOG_NOTICE,\
-      GRN_CTX_FIN, 0, 0, 0, 0, {0}, NULL, NULL, NULL, NULL, NULL }
+    GRN_CTX_FIN, 0, 0, 0, 0, {0}, NULL, NULL, NULL, NULL, NULL }
 
 #define GRN_CTX_CLOSED(ctx) ((ctx)->stat == GRN_CTX_FIN)
 
 #ifdef USE_EXACT_ALLOC_COUNT
-#define GRN_ADD_ALLOC_COUNT(count) \
-{ \
+#define GRN_ADD_ALLOC_COUNT(count) do { \
   uint32_t alloced; \
   GRN_ATOMIC_ADD_EX(&alloc_count, count, alloced); \
-}
+} while (0)
 #else /* USE_EXACT_ALLOC_COUNT */
-#define GRN_ADD_ALLOC_COUNT(count) \
-{ \
+#define GRN_ADD_ALLOC_COUNT(count) do { \
   alloc_count += count; \
-}
+} while (0)
 #endif
 
 grn_ctx grn_gctx = GRN_CTX_INITIALIZER(GRN_ENC_DEFAULT);
@@ -420,8 +418,10 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->encoding = ctx->encoding;
   ctx->impl->lifoseg = -1;
   ctx->impl->currseg = -1;
+  CRITICAL_SECTION_INIT(ctx->impl->lock);
   if (!(ctx->impl->values = grn_array_create(ctx, NULL, sizeof(grn_db_obj *),
                                              GRN_ARRAY_TINY))) {
+    CRITICAL_SECTION_FIN(ctx->impl->lock);
     grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
     ctx->impl = NULL;
     return;
@@ -430,11 +430,11 @@ grn_ctx_impl_init(grn_ctx *ctx)
                                          sizeof(grn_io *),
                                          GRN_OBJ_KEY_VAR_SIZE|GRN_HASH_TINY))) {
     grn_array_close(ctx, ctx->impl->values);
+    CRITICAL_SECTION_FIN(ctx->impl->lock);
     grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
     ctx->impl = NULL;
     return;
   }
-  CRITICAL_SECTION_INIT(ctx->impl->lock);
   ctx->impl->db = NULL;
 
   ctx->impl->expr_vars = grn_hash_create(ctx, NULL, sizeof(grn_id), sizeof(grn_obj *), 0);
@@ -701,7 +701,8 @@ default_logger_func(int level, const char *time, const char *title,
 static grn_logger_info default_logger = {
   GRN_LOG_DEFAULT_LEVEL,
   GRN_LOG_TIME|GRN_LOG_MESSAGE,
-  default_logger_func
+  default_logger_func,
+  NULL
 };
 
 static const grn_logger_info *grn_logger = &default_logger;
@@ -741,6 +742,31 @@ grn_log_reopen(grn_ctx *ctx)
 static grn_obj grn_true_, grn_false_, grn_null_;
 grn_obj *grn_true, *grn_false, *grn_null;
 
+static void
+check_overcommit_memory(grn_ctx *ctx)
+{
+  FILE *file;
+  int value;
+  file = fopen("/proc/sys/vm/overcommit_memory", "r");
+  if (!file) { return; }
+  value = fgetc(file);
+  if (value != '1') {
+    GRN_LOG(ctx, GRN_LOG_NOTICE,
+            "vm.overcommit_memory kernel parameter should be 1: <%c>", value);
+    GRN_LOG(ctx, GRN_LOG_NOTICE,
+            "Some processings with vm.overcommit_memory != 1 "
+            "may break DB under low memory condition.");
+    GRN_LOG(ctx, GRN_LOG_NOTICE,
+            "To set vm.overcommit_memory to 1");
+    GRN_LOG(ctx, GRN_LOG_NOTICE,
+            "add 'vm.overcommit_memory = 1' to /etc/sysctl.conf and "
+            "restart your system or");
+    GRN_LOG(ctx, GRN_LOG_NOTICE,
+            "run 'sudo /sbin/sysctl vm.overcommit_memory=1' command.");
+  }
+  fclose(file);
+}
+
 grn_rc
 grn_init(void)
 {
@@ -837,6 +863,7 @@ grn_init(void)
   */
   grn_cache_init();
   GRN_LOG(ctx, GRN_LOG_NOTICE, "grn_init");
+  check_overcommit_memory(ctx);
   return rc;
 }
 
@@ -1125,11 +1152,11 @@ get_command_version(grn_ctx *ctx, const char *p, const char *pe)
   }
 }
 
-#define INDEX_HTML "index.html"
-#define OUTPUT_TYPE "output_type"
-#define COMMAND_VERSION "command_version"
-#define EXPR_MISSING "expr_missing"
-#define OUTPUT_TYPE_LEN (sizeof(OUTPUT_TYPE) - 1)
+#define INDEX_HTML          "index.html"
+#define OUTPUT_TYPE         "output_type"
+#define COMMAND_VERSION     "command_version"
+#define EXPR_MISSING        "expr_missing"
+#define OUTPUT_TYPE_LEN     (sizeof(OUTPUT_TYPE) - 1)
 #define COMMAND_VERSION_LEN (sizeof(COMMAND_VERSION) - 1)
 
 static inline int
