@@ -12,13 +12,14 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef GRN_DAT_COMMON_HPP_
 #define GRN_DAT_COMMON_HPP_
 
 #ifndef _MSC_VER
+ #include <stddef.h>
  #include <stdint.h>
 #endif  // _MSC_VER
 
@@ -29,6 +30,18 @@
  #include <iostream>
 #endif  // _DEBUG
 
+#ifndef GRN_DAT_API
+ #ifdef WIN32
+  #ifdef GRN_DAT_EXPORT
+   #define GRN_DAT_API __declspec(dllexport)
+  #else  // GRN_DAT_EXPORT
+   #define GRN_DAT_API __declspec(dllimport)
+  #endif  // GRN_DAT_EXPORT
+ #else  // WIN32
+  #define GRN_DAT_API
+ #endif  // WIN32
+#endif  // GRN_DAT_API
+
 namespace grn {
 namespace dat {
 
@@ -157,15 +170,18 @@ enum ErrorCode {
 class Exception : public std::exception {
  public:
   Exception() throw()
-      : file_(""),
+      : std::exception(),
+        file_(""),
         line_(-1),
         what_("") {}
   Exception(const char *file, int line, const char *what) throw()
-      : file_(file),
+      : std::exception(),
+        file_(file),
         line_(line),
         what_((what != NULL) ? what : "") {}
   Exception(const Exception &ex) throw()
-      : file_(ex.file_),
+      : std::exception(ex),
+        file_(ex.file_),
         line_(ex.line_),
         what_(ex.what_) {}
   virtual ~Exception() throw() {}
