@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2011 Brazil
+/* Copyright(C) 2009-2012 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -70,6 +70,30 @@
 #  define GRN_VAR extern
 #endif
 
+#ifdef HAVE_OPEN
+# define GRN_OPEN(pathname, ...) open(pathname, __VA_ARGS__)
+#else
+# define GRN_OPEN(pathname, ...) _open(pathname, __VA_ARGS__)
+#endif /* HAVE_OPEN */
+
+#ifdef HAVE_CLOSE
+# define GRN_CLOSE(fd) close(fd)
+#else
+# define GRN_CLOSE(fd) _close(fd)
+#endif /* HAVE_CLOSE */
+
+#ifdef HAVE_READ
+# define GRN_READ(fd, buf, count) read(fd, buf, count)
+#else
+# define GRN_READ(fd, buf, count) _read(fd, buf, count)
+#endif /* HAVE_READ */
+
+#ifdef HAVE_WRITE
+# define GRN_WRITE(fd, buf, count) write(fd, buf, count)
+#else
+# define GRN_WRITE(fd, buf, count) _write(fd, buf, count)
+#endif /* HAVE_WRITE */
+
 #ifdef WIN32
 
 #if defined(__GNUC__) && !defined(WINVER)
@@ -93,27 +117,22 @@
 #define inline _inline
 #endif
 
-#define snprintf _snprintf
+#define snprintf(str, size, ...) _snprintf(str, size, __VA_ARGS__)
 #if _MSC_VER < 1500
-  #define vsnprintf _vsnprintf
+# define vsnprintf(str, size, format, ap) _vsnprintf(str, size, format, ap)
 #endif /* _MSC_VER < 1500 */
-#define unlink _unlink
-#define open _open
-#define lseek _lseek
-#define read _read
-#define getpid _getpid
+#define unlink(pathname) _unlink(pathname)
+#define lseek(fd, offset, whence) _lseek(fd, offset, whence)
+#define getpid() _getpid()
 #if !defined(__GNUC__) && _MSC_VER < 1400
-# define fstat _fstat
+# define fstat(fd, buf) _fstat(fd, buf)
 #endif /* !defined(__GNUC__) && _MSC_VER < 1400 */
-#define write _write
-#define close _close
 #define usleep(x) Sleep((x) / 1000)
-#define sleep(x) Sleep((x) * 1000)
+#define sleep(x)  Sleep((x) * 1000)
 #if !defined(strcasecmp)
-#  define strcasecmp stricmp
+#  define strcasecmp(s1, s2) stricmp(s1, s2)
 #endif /* !defined(strcasecmp) */
 
-
 #ifdef __GNUC__
 #include <stdint.h>
 #else
@@ -133,16 +152,20 @@
 
 #undef MSG_WAITALL
 #define MSG_WAITALL 0 /* before Vista, not supported... */
-#define fpclassify _fpclass
-#define CASE_FP_NAN case _FPCLASS_SNAN: case _FPCLASS_QNAN:
-#define CASE_FP_INFINITE case _FPCLASS_NINF: case _FPCLASS_PINF:
 #define SHUT_RDWR SD_BOTH
 
-#define islessgreater(x,y)  (!_isnan(x) && !_isnan(y) && ((x) < (y) || (x) > (y)))
-#define isless(x,y)         (!_isnan(x) && !_isnan(y) && ((x) < (y)))
-#define isgreater(x,y)      (!_isnan(x) && !_isnan(y) && ((x) > (y)))
+#ifdef HAVE_FPCLASSIFY
+# define CASE_FP_NAN case FP_NAN:
+# define CASE_FP_INFINITE case FP_INFINITE:
+#else
+# define HAVE_FPCLASSIFY 1
+# define fpclassify _fpclass
+# define CASE_FP_NAN case _FPCLASS_SNAN: case _FPCLASS_QNAN:
+# define CASE_FP_INFINITE case _FPCLASS_NINF: case _FPCLASS_PINF:
+#endif /* HAVE_FPCLASSIFY */
+
 typedef SOCKET grn_sock;
-#define grn_sock_close closesocket
+#define grn_sock_close(sock) closesocket(sock)
 
 #define CALLBACK __stdcall
 
@@ -151,6 +174,7 @@ typedef SOCKET grn_sock;
 #include <sys/timeb.h>
 #include <errno.h>
 #endif
+
 #else /* WIN32 */
 
 #define GROONGA_API
@@ -173,23 +197,21 @@ typedef char int_least8_t;
 typedef unsigned char uint_least8_t;
 # endif /* UINT_LEAST8_MAX */
 typedef int grn_sock;
-# define grn_sock_close close
+# define grn_sock_close(sock) close(sock)
 # define CALLBACK
 
-# if (defined(__sun) && defined(__SVR4)) /* SUN */
-#  include <ieeefp.h>
-#  define fpclassify fpclass
-#  define CASE_FP_NAN case FP_SNAN: case FP_QNAN:
-#  define CASE_FP_INFINITE case FP_NINF: case FP_PINF:
-#  ifdef __GNUC__
-#   define isgreater(x,y) ((x) > (y))
-#   define isless(x,y) ((x) < (y))
-#   define islessgreater(x,y) ((x) != (y))
-#  endif /* __GNUC__ */
-# else  /* SUN */
+# ifdef HAVE_FPCLASSIFY
 #  define CASE_FP_NAN case FP_NAN:
 #  define CASE_FP_INFINITE case FP_INFINITE:
-# endif /* SUN */
+# else
+#  if (defined(__sun) && defined(__SVR4)) /* SUN */
+#   define HAVE_FPCLASSIFY 1
+#   include <ieeefp.h>
+#   define fpclassify fpclass
+#   define CASE_FP_NAN case FP_SNAN: case FP_QNAN:
+#   define CASE_FP_INFINITE case FP_NINF: case FP_PINF:
+#  endif /* SUN */
+# endif /* HAVE_FPCLASSIFY */
 
 #endif /* WIN32 */
 
@@ -232,50 +254,51 @@ typedef int grn_sock;
 #ifdef HAVE_PTHREAD_H
 #include <pthread.h>
 typedef pthread_t grn_thread;
-#define THREAD_CREATE(thread,func,arg) (pthread_create(&(thread), NULL, (func), (arg)))
-#define THREAD_JOIN(thread) pthread_join(thread, NULL);
+#define THREAD_CREATE(thread,func,arg) \
+  (pthread_create(&(thread), NULL, (func), (arg)))
+#define THREAD_JOIN(thread) (pthread_join(thread, NULL))
 typedef pthread_mutex_t grn_mutex;
-#define MUTEX_INIT(m) pthread_mutex_init(&m, NULL)
-#define MUTEX_LOCK(m) pthread_mutex_lock(&m)
+#define MUTEX_INIT(m)   pthread_mutex_init(&m, NULL)
+#define MUTEX_LOCK(m)   pthread_mutex_lock(&m)
 #define MUTEX_UNLOCK(m) pthread_mutex_unlock(&m)
 #define MUTEX_FIN(m)
 typedef pthread_mutex_t grn_critical_section;
-#define CRITICAL_SECTION_INIT(cs) pthread_mutex_init(&(cs), NULL)
+#define CRITICAL_SECTION_INIT(cs)  pthread_mutex_init(&(cs), NULL)
 #define CRITICAL_SECTION_ENTER(cs) pthread_mutex_lock(&(cs))
 #define CRITICAL_SECTION_LEAVE(cs) pthread_mutex_unlock(&(cs))
 #define CRITICAL_SECTION_FIN(cs)
 
 typedef pthread_cond_t grn_cond;
-#define COND_INIT(c) pthread_cond_init(&c, NULL)
+#define COND_INIT(c)   pthread_cond_init(&c, NULL)
 #define COND_SIGNAL(c) pthread_cond_signal(&c)
 #define COND_WAIT(c,m) pthread_cond_wait(&c, &m)
 
 typedef pthread_key_t grn_thread_key;
-#define THREAD_KEY_CREATE pthread_key_create
-#define THREAD_KEY_DELETE pthread_key_delete
-#define THREAD_SETSPECIFIC pthread_setspecific
-#define THREAD_GETSPECIFIC pthread_getspecific
+#define THREAD_KEY_CREATE(key, destr)  pthread_key_create(key, destr)
+#define THREAD_KEY_DELETE(key)         pthread_key_delete(key)
+#define THREAD_SETSPECIFIC(key, value) pthread_setspecific(key, value)
+#define THREAD_GETSPECIFIC(key)        pthread_getspecific(key)
 
 #if USE_UYIELD
   extern int grn_uyield_count;
-  #define GRN_TEST_YIELD() \
-    do {\
-      if (((++grn_uyield_count) & (0x20 - 1)) == 0) {\
-        sched_yield();\
-        if(grn_uyield_count > 0x1000) {\
-          grn_uyield_count = (uint32_t)time(NULL) % 0x1000;\
-        }\
+  #define GRN_TEST_YIELD() do {\
+    if (((++grn_uyield_count) & (0x20 - 1)) == 0) {\
+      sched_yield();\
+      if(grn_uyield_count > 0x1000) {\
+        grn_uyield_count = (uint32_t)time(NULL) % 0x1000;\
       }\
-    } while (0)
+    }\
+  } while (0)
+
   #undef assert
-  #define assert(assert_expr) \
-    do {\
-      if (!(assert_expr)){\
-        fprintf(stderr, "assertion failed: %s\n", #assert_expr);\
-        abort();\
-      }\
-      GRN_TEST_YIELD();\
-    } while (0)
+  #define assert(assert_expr) do {\
+    if (!(assert_expr)){\
+      fprintf(stderr, "assertion failed: %s\n", #assert_expr);\
+      abort();\
+    }\
+    GRN_TEST_YIELD();\
+  } while (0)
+
   #define if(if_cond) \
     if ((((++grn_uyield_count) & (0x100 - 1)) != 0 || (sched_yield() * 0) == 0) && (if_cond))
   #define while(while_cond) \
@@ -285,9 +308,7 @@ typedef pthread_key_t grn_thread_key;
   #define sched_yield() usleep(1000 * 20)
   #endif
 #else /* USE_UYIELD */
-  #define GRN_TEST_YIELD() \
-    do { \
-    } while (0)
+  #define GRN_TEST_YIELD() do {} while (0)
 #endif /* USE_UYIELD */
 
 #else /* HAVE_PTHREAD_H */
@@ -301,18 +322,20 @@ typedef int grn_thread_key;
 
 #ifdef WIN32
 typedef uintptr_t grn_thread;
-#define THREAD_CREATE(thread,func,arg) (((thread)=_beginthreadex(NULL, 0, (func), (arg), 0, NULL)) == NULL)
-#define THREAD_JOIN(thread) WaitForSingleObject((thread), INFINITE);
+#define THREAD_CREATE(thread,func,arg) \
+  (((thread)=_beginthreadex(NULL, 0, (func), (arg), 0, NULL)) == NULL)
+#define THREAD_JOIN(thread) \
+  (WaitForSingleObject((thread), INFINITE) == WAIT_FAILED)
 typedef HANDLE grn_mutex;
-#define MUTEX_INIT(m) ((m) = CreateMutex(0, FALSE, NULL))
-#define MUTEX_LOCK(m) WaitForSingleObject((m), INFINITE)
+#define MUTEX_INIT(m)   ((m) = CreateMutex(0, FALSE, NULL))
+#define MUTEX_LOCK(m)   WaitForSingleObject((m), INFINITE)
 #define MUTEX_UNLOCK(m) ReleaseMutex(m)
-#define MUTEX_FIN(m) CloseHandle(m)
+#define MUTEX_FIN(m)    CloseHandle(m)
 typedef CRITICAL_SECTION grn_critical_section;
-#define CRITICAL_SECTION_INIT(cs) InitializeCriticalSection(&(cs))
+#define CRITICAL_SECTION_INIT(cs)  InitializeCriticalSection(&(cs))
 #define CRITICAL_SECTION_ENTER(cs) EnterCriticalSection(&(cs))
 #define CRITICAL_SECTION_LEAVE(cs) LeaveCriticalSection(&(cs))
-#define CRITICAL_SECTION_FIN(cs) DeleteCriticalSection(&(cs))
+#define CRITICAL_SECTION_FIN(cs)   DeleteCriticalSection(&(cs))
 
 typedef struct
 {
@@ -322,13 +345,13 @@ typedef struct
   HANDLE waiters_done_;
 } grn_cond;
 
-#define COND_INIT(c) { \
+#define COND_INIT(c) do { \
   (c).waiters_count_ = 0; \
   (c).sema_ = CreateSemaphore(NULL, 0, 0x7fffffff, NULL); \
   MUTEX_INIT((c).waiters_count_lock_); \
-}
+} while (0)
 
-#define COND_SIGNAL(c) { \
+#define COND_SIGNAL(c) do { \
   MUTEX_LOCK((c).waiters_count_lock_); \
   { \
     int have_waiters = (c).waiters_count_ > 0; \
@@ -337,8 +360,9 @@ typedef struct
       ReleaseSemaphore((c).sema_, 1, 0); \
     } \
   } \
-}
-#define COND_WAIT(c,m) { \
+} while (0)
+
+#define COND_WAIT(c,m) do { \
   MUTEX_LOCK((c).waiters_count_lock_); \
   (c).waiters_count_++; \
   MUTEX_UNLOCK((c).waiters_count_lock_); \
@@ -347,21 +371,23 @@ typedef struct
   (c).waiters_count_--; \
   MUTEX_UNLOCK((c).waiters_count_lock_); \
   WaitForSingleObject((m), INFINITE); \
-}
+} while (0)
 
 #else /* WIN32 */
 /* todo */
 typedef int grn_cond;
-#define COND_INIT(c) ((c) = 0)
+#define COND_INIT(c)   ((c) = 0)
 #define COND_SIGNAL(c)
-#define COND_WAIT(c,m) do { MUTEX_UNLOCK(m); usleep(1000); MUTEX_LOCK(m); } while (0)
+#define COND_WAIT(c,m) do { \
+  MUTEX_UNLOCK(m); \
+  usleep(1000); \
+  MUTEX_LOCK(m); \
+} while (0)
 /* todo : must be enhanced! */
 
 #endif /* WIN32 */
 
-#define GRN_TEST_YIELD() \
-  do { \
-  } while (0)
+#define GRN_TEST_YIELD() do {} while (0)
 
 #endif /* HAVE_PTHREAD_H */
 
@@ -384,29 +410,29 @@ typedef int grn_cond;
   __asm__ __volatile__ ("\tmovl %1,%%eax\n\tbsr %%eax,%0\n\tcmovz %%eax,%0\n" : "=r"(r) : "r"(v) : "%eax")
 # elif (defined(__PPC__) || defined(__ppc__)) /* ATOMIC ADD */
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
-  __asm__ __volatile__ ("\n1:\n\tlwarx %0, 0, %1\n\tadd %0, %0, %2\n\tstwcx. %0, 0, %1\n\tbne- 1b\n\tsub %0, %0, %2" : "=&r" (r) : "r" (p), "r" (i) : "cc", "memory");
+  __asm__ __volatile__ ("\n1:\n\tlwarx %0, 0, %1\n\tadd %0, %0, %2\n\tstwcx. %0, 0, %1\n\tbne- 1b\n\tsub %0, %0, %2" : "=&r" (r) : "r" (p), "r" (i) : "cc", "memory")
 /* todo */
-#  define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-#  define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+#  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+#  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # elif (defined(__sun) && defined(__SVR4)) /* ATOMIC ADD */
 #  include <atomic.h>
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
-  r = atomic_add_32_nv(p, i) - i
+  (r = atomic_add_32_nv(p, i) - i)
 /* todo */
-#  define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-#  define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+#  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+#  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # else /* ATOMIC ADD */
 /* todo */
-#  define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-#  define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+#  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+#  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 # endif /* ATOMIC ADD */
 
 # ifdef __i386__ /* ATOMIC 64BIT SET */
 #  define GRN_SET_64BIT(p,v) \
-  __asm__ __volatile__ ("\txchgl %%esi, %%ebx\n1:\n\tmovl (%0), %%eax\n\tmovl 4(%0), %%edx\n\tlock; cmpxchg8b (%0)\n\tjnz 1b\n\txchgl %%ebx, %%esi" : : "D"(p), "S"(*(((uint32_t *)&(v))+0)), "c"(*(((uint32_t *)&(v))+1)) : "ax", "dx", "memory");
+  __asm__ __volatile__ ("\txchgl %%esi, %%ebx\n1:\n\tmovl (%0), %%eax\n\tmovl 4(%0), %%edx\n\tlock; cmpxchg8b (%0)\n\tjnz 1b\n\txchgl %%ebx, %%esi" : : "D"(p), "S"(*(((uint32_t *)&(v))+0)), "c"(*(((uint32_t *)&(v))+1)) : "ax", "dx", "memory")
 # elif defined(__x86_64__) /* ATOMIC 64BIT SET */
 #  define GRN_SET_64BIT(p,v) \
-  *(p) = (v);
+  (*(p) = (v))
 # elif (defined(__sun) && defined(__SVR4)) /* ATOMIC 64BIT SET */
 /* todo */
 #  define GRN_SET_64BIT(p,v) \
@@ -414,22 +440,21 @@ typedef int grn_cond;
 # endif /* ATOMIC 64BIT SET */
 
 # ifdef HAVE_MKOSTEMP
-#  define GRN_MKOSTEMP mkostemp
+#  define GRN_MKOSTEMP(template,flags,mode) mkostemp(template,flags)
 # else /* HAVE_MKOSTEMP */
-#  define GRN_MKOSTEMP(template,flags) \
-  (mktemp(template), open((template),flags))
+#  define GRN_MKOSTEMP(template,flags,mode) \
+  (mktemp(template), GRN_OPEN((template),flags,mode))
 # endif /* HAVE_MKOSTEMP */
 
 #elif (defined(WIN32) || defined (_WIN64)) /* __GNUC__ */
 
 # define GRN_ATOMIC_ADD_EX(p,i,r) \
-  (r) = (uint32_t)InterlockedExchangeAdd((int32_t *)(p), (int32_t)(i));
+  ((r) = (uint32_t)InterlockedExchangeAdd((int32_t *)(p), (int32_t)(i)))
 # if defined(_WIN64) /* ATOMIC 64BIT SET */
 #  define GRN_SET_64BIT(p,v) \
-  *(p) = (v);
+  (*(p) = (v))
 # else /* ATOMIC 64BIT SET */
-#  define GRN_SET_64BIT(p,v) \
-{\
+#  define GRN_SET_64BIT(p,v) do {\
   uint32_t v1, v2; \
   uint64_t *p2= (p); \
   v1 = *(((uint32_t *)&(v))+0);\
@@ -442,16 +467,16 @@ typedef int grn_cond;
   __asm  mov edx, dword ptr [esi + 4] \
   __asm  lock cmpxchg8b qword ptr [esi] \
   __asm  jnz  _set_loop \
-}\
+} while (0)
 /* TODO: use _InterlockedCompareExchange64 or inline asm */
 # endif /* ATOMIC 64BIT SET */
 
 /* todo */
-# define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-# define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+# define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+# define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 
-# define GRN_MKOSTEMP(template,flags) \
-  (mktemp(template), open((template),((flags)|O_BINARY)))
+# define GRN_MKOSTEMP(template,flags,mode) \
+  (mktemp(template), GRN_OPEN((template),((flags)|O_BINARY),mode))
 
 #else /* __GNUC__ */
 
@@ -459,20 +484,20 @@ typedef int grn_cond;
 #  define __FUNCTION__ ""
 #  include <atomic.h>
 #  define GRN_ATOMIC_ADD_EX(p,i,r) \
-  r = atomic_add_32_nv(p, i) - i
+  (r = atomic_add_32_nv(p, i) - i)
 /* todo */
-#  define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-#  define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+#  define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+#  define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 /* todo */
 #  define GRN_SET_64BIT(p,v) \
   (void)atomic_swap_64(p, v)
 # endif /* ATOMIC ADD */
 /* todo */
-# define GRN_BIT_SCAN_REV(v,r)   for (r = 31; r && !((1 << r) & v); r--)
-# define GRN_BIT_SCAN_REV0 GRN_BIT_SCAN_REV
+# define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+# define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 
-# define GRN_MKOSTEMP(template,flags) \
-  (mktemp(template), open((template),flags))
+# define GRN_MKOSTEMP(template,flags,mode) \
+  (mktemp(template), GRN_OPEN((template),flags,mode))
 
 #endif /* __GNUC__ */
 
@@ -497,42 +522,37 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 #endif /* __GNUC__ */
 
 #ifdef WORDS_BIGENDIAN
-#define grn_hton(buf,key,size) \
-{\
+#define grn_hton(buf,key,size) do {\
   uint32_t size_ = (uint32_t)size;\
   uint8_t *buf_ = (uint8_t *)buf;\
   uint8_t *key_ = (uint8_t *)key;\
   while (size_--) { *buf_++ = *key_++; }\
-}
-#define grn_ntohi(buf,key,size) \
-{\
+} while (0)
+#define grn_ntohi(buf,key,size) do {\
   uint32_t size_ = (uint32_t)size;\
   uint8_t *buf_ = (uint8_t *)buf;\
   uint8_t *key_ = (uint8_t *)key;\
   if (size_) { *buf_++ = 0x80 ^ *key_++; size_--; }\
   while (size_) { *buf_++ = *key_++; size_--; }\
-}
+} while (0)
 #else /* WORDS_BIGENDIAN */
-#define grn_hton(buf,key,size) \
-{\
+#define grn_hton(buf,key,size) do {\
   uint32_t size_ = (uint32_t)size;\
   uint8_t *buf_ = (uint8_t *)buf;\
   uint8_t *key_ = (uint8_t *)key + size;\
   while (size_--) { *buf_++ = *(--key_); }\
-}
-#define grn_ntohi(buf,key,size) \
-{\
+} while (0)
+#define grn_ntohi(buf,key,size) do {\
   uint32_t size_ = (uint32_t)size;\
   uint8_t *buf_ = (uint8_t *)buf;\
   uint8_t *key_ = (uint8_t *)key + size;\
   while (size_ > 1) { *buf_++ = *(--key_); size_--; }\
   if (size_) { *buf_ = 0x80 ^ *(--key_); } \
-}
+} while (0)
 #endif /* WORDS_BIGENDIAN */
-#define grn_ntoh grn_hton
+#define grn_ntoh(buf,key,size) grn_hton(buf,key,size)
 
-#define grn_gton(keybuf,key,size)\
-{\
+#define grn_gton(keybuf,key,size) do {\
   uint8_t *keybuf_ = keybuf;\
   const void *key_ = key;\
   int la = ((grn_geo_point *)key_)->latitude;\
@@ -546,10 +566,9 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
             (((la >> i) & 2) << 2) + (((lo >> i) & 2) << 1) +\
             (((la >> i) & 1) << 1) + (((lo >> i) & 1) << 0));\
   }\
-}
+} while (0)
 
-#define grn_ntog(keybuf,key,size)\
-{\
+#define grn_ntog(keybuf,key,size) do {\
   uint8_t *keybuf_ = keybuf;\
   uint8_t *key_ = key;\
   uint32_t size_ = size;\
@@ -566,7 +585,7 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
   }\
   ((grn_geo_point *)keybuf_)->latitude = la;\
   ((grn_geo_point *)keybuf_)->longitude = lo;\
-}
+} while (0)
 
 #ifdef USE_FUTEX
 #include <linux/futex.h>
@@ -589,8 +608,7 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
   }\
 } while(0)
 
-#define GRN_FUTEX_WAKE(p) \
-  syscall(SYS_futex, p, FUTEX_WAKE, 1)
+#define GRN_FUTEX_WAKE(p) syscall(SYS_futex, p, FUTEX_WAKE, 1)
 #else /* USE_FUTEX */
 #define GRN_FUTEX_WAIT(p) usleep(1000)
 #define GRN_FUTEX_WAKE(p)
