@@ -16,7 +16,6 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "hash.h"
-#include "pat.h"
 #include "output.h"
 #include <string.h>
 #include <limits.h>
@@ -25,61 +24,159 @@
 
 /* grn_tiny_array */
 
-void
-grn_tiny_array_init(grn_ctx *ctx, grn_tiny_array *a, uint16_t element_size, uint16_t flags)
+inline static int
+grn_tiny_array_get_block_id(grn_tiny_array *array, grn_id id)
 {
-  a->ctx = ctx;
-  a->element_size = element_size;
-  a->flags = flags;
-  a->max = 0;
-  if (flags & GRN_TINY_ARRAY_THREADSAFE) { CRITICAL_SECTION_INIT(a->lock); }
-  memset(a->elements, 0, sizeof(a->elements));
+  int most_significant_one_bit_offset;
+  GRN_BIT_SCAN_REV(id, most_significant_one_bit_offset);
+  return most_significant_one_bit_offset >> GRN_TINY_ARRAY_W;
+}
+
+inline static void *
+grn_tiny_array_at_inline(grn_tiny_array *array, grn_id id)
+{
+  int block_id;
+  void **block;
+  size_t offset;
+  if (!id) {
+    return NULL;
+  }
+  block_id = grn_tiny_array_get_block_id(array, id);
+  block = &array->elements[block_id];
+  offset = GRN_TINY_ARRAY_R(block_id);
+  if (!*block) {
+    grn_ctx * const ctx = array->ctx;
+    if (array->flags & GRN_TINY_ARRAY_THREADSAFE) {
+      CRITICAL_SECTION_ENTER(array->lock);
+    }
+    if (!*block) {
+      const size_t block_size =
+          GRN_TINY_ARRAY_S * offset * array->element_size;
+      if (array->flags & GRN_TINY_ARRAY_USE_MALLOC) {
+        if (array->flags & GRN_TINY_ARRAY_CLEAR) {
+          *block = GRN_CALLOC(block_size);
+        } else {
+          *block = GRN_MALLOC(block_size);
+        }
+      } else {
+        *block = GRN_CTX_ALLOC(ctx, block_size);
+      }
+    }
+    if (array->flags & GRN_TINY_ARRAY_THREADSAFE) {
+      CRITICAL_SECTION_LEAVE(array->lock);
+    }
+    if (!*block) {
+      return NULL;
+    }
+  }
+  if (id > array->max) {
+    array->max = id;
+  }
+  return (byte *)*block + (id - offset) * array->element_size;
+}
+
+inline static void *
+grn_tiny_array_next(grn_tiny_array *array)
+{
+  return grn_tiny_array_at_inline(array, array->max + 1);
+}
+
+inline static grn_bool
+grn_tiny_array_bit_at(grn_tiny_array *array, grn_id offset)
+{
+  uint8_t * const ptr =
+      (uint8_t *)grn_tiny_array_at_inline(array, (offset >> 3) + 1);
+  return ptr ? ((*ptr >> (offset & 7)) & 1) : 0;
+}
+
+inline static void
+grn_tiny_array_bit_on(grn_tiny_array *array, grn_id offset)
+{
+  uint8_t * const ptr =
+      (uint8_t *)grn_tiny_array_at_inline(array, (offset >> 3) + 1);
+  if (ptr) {
+    *ptr |= 1 << (offset & 7);
+  }
+}
+
+inline static void
+grn_tiny_array_bit_off(grn_tiny_array *array, grn_id offset)
+{
+  uint8_t * const ptr =
+      (uint8_t *)grn_tiny_array_at_inline(array, (offset >> 3) + 1);
+  if (ptr) {
+    *ptr &= ~(1 << (offset & 7));
+  }
+}
+
+inline static void
+grn_tiny_array_bit_flip(grn_tiny_array *array, grn_id offset)
+{
+  uint8_t * const ptr =
+      (uint8_t *)grn_tiny_array_at_inline(array, (offset >> 3) + 1);
+  if (ptr) {
+    *ptr ^= 1 << (offset & 7);
+  }
 }
 
 void
-grn_tiny_array_fin(grn_tiny_array *a)
+grn_tiny_array_init(grn_ctx *ctx, grn_tiny_array *array,
+                    uint16_t element_size, uint16_t flags)
 {
-  int i;
-  grn_ctx *ctx = a->ctx;
-  for (i = 0; i < GRN_TINY_ARRAY_N; i++) {
-    void **e = &a->elements[i];
-    if (*e) {
-      if (a->flags & GRN_TINY_ARRAY_USE_MALLOC) {
-        GRN_FREE(*e);
+  array->ctx = ctx;
+  array->element_size = element_size;
+  array->flags = flags;
+  array->max = 0;
+  if (flags & GRN_TINY_ARRAY_THREADSAFE) {
+    CRITICAL_SECTION_INIT(array->lock);
+  }
+  memset(array->elements, 0, sizeof(array->elements));
+}
+
+void
+grn_tiny_array_fin(grn_tiny_array *array)
+{
+  int block_id;
+  grn_ctx * const ctx = array->ctx;
+  for (block_id = 0; block_id < GRN_TINY_ARRAY_N; block_id++) {
+    void ** const block = &array->elements[block_id];
+    if (*block) {
+      if (array->flags & GRN_TINY_ARRAY_USE_MALLOC) {
+        GRN_FREE(*block);
       } else {
-        GRN_CTX_FREE(ctx, *e);
+        GRN_CTX_FREE(ctx, *block);
       }
-      *e = NULL;
+      *block = NULL;
     }
   }
 }
 
 void *
-grn_tiny_array_at(grn_tiny_array *a, grn_id id)
+grn_tiny_array_at(grn_tiny_array *array, grn_id id)
 {
-  void *e;
-  GRN_TINY_ARRAY_AT(a, id, e);
-  return e;
+  return grn_tiny_array_at_inline(array, id);
 }
 
 grn_id
-grn_tiny_array_id(grn_tiny_array *a, void *p)
+grn_tiny_array_id(grn_tiny_array *array, const void *element_address)
 {
-  uint32_t i, s, n;
-  uintptr_t o, p_ = (uintptr_t) p;
-  for (i = 0, s = 0; i < GRN_TINY_ARRAY_N; i++, s += n) {
-    n = GRN_TINY_ARRAY_S * GRN_TINY_ARRAY_R(i);
-    o = (uintptr_t) a->elements[i];
-    if (o <= p_ && p_ < o + n * a->element_size) {
-      return s + ((p_ - o) / a->element_size);
+  const byte * const ptr = (const byte *)element_address;
+  uint32_t block_id, offset = 1;
+  for (block_id = 0; block_id < GRN_TINY_ARRAY_N; block_id++) {
+    const uint32_t block_size = GRN_TINY_ARRAY_S * GRN_TINY_ARRAY_R(block_id);
+    const byte * const block = (const byte *)array->elements[block_id];
+    if (block && block <= ptr &&
+        ptr < (block + block_size * array->element_size)) {
+      return offset + ((ptr - block) / array->element_size);
     }
+    offset += block_size;
   }
   return GRN_ID_NIL;
 }
 
 /* grn_array */
 
-#define GRN_ARRAY_HEADER_SIZE 0x9000
+#define GRN_ARRAY_HEADER_SIZE  0x9000
 #define GRN_ARRAY_SEGMENT_SIZE 0x400000
 
 struct grn_array_header {
@@ -100,30 +197,39 @@ enum {
 
 #define IO_ARRAYP(array) ((array)->io)
 
-#define ARRAY_ENTRY_AT_(array,id,value,addp) {\
-  int flags = addp;\
-  GRN_IO_ARRAY_AT(array->io, array_seg_value, id, &flags, value);\
+inline static void *
+grn_array_io_entry_at(grn_ctx *ctx, grn_array *array, grn_id id, int flags)
+{
+  void *value;
+  GRN_IO_ARRAY_AT(array->io, array_seg_value, id, &flags, value);
+  return value;
 }
 
-#define ARRAY_ENTRY_AT(array,id,value,addp) {\
-  if (IO_ARRAYP(array)) {\
-    ARRAY_ENTRY_AT_(array, id, value, addp);\
-  } else {\
-    GRN_TINY_ARRAY_AT(&array->a, id, value);\
-  }\
+inline static void *
+grn_array_entry_at(grn_ctx *ctx, grn_array *array, grn_id id, int flags)
+{
+  if (IO_ARRAYP(array)) {
+    return grn_array_io_entry_at(ctx, array, id, flags);
+  } else {
+    return grn_tiny_array_at_inline(&array->a, id);
+  }
 }
 
-#define ARRAY_BITMAP_AT(array,id,value) {\
-  if (IO_ARRAYP(array)) {\
-    GRN_IO_ARRAY_BIT_AT((array)->io, array_seg_bitmap, (id), (value));\
-  } else {\
-    GRN_TINY_ARRAY_BIT_AT(&(array)->bitmap, (id), (value));\
-  }\
+inline static grn_bool
+grn_array_bitmap_at(grn_ctx *ctx, grn_array *array, grn_id id)
+{
+  if (IO_ARRAYP(array)) {
+    grn_bool value;
+    GRN_IO_ARRAY_BIT_AT(array->io, array_seg_bitmap, id, value);
+    return value;
+  } else {
+    return grn_tiny_array_bit_at(&array->bitmap, id);
+  }
 }
 
 static grn_rc
 tiny_array_init(grn_ctx *ctx, grn_array *array, const char *path,
-               uint32_t value_size, uint32_t flags)
+                uint32_t value_size, uint32_t flags)
 {
   if (path) {
     ERR(GRN_INVALID_ARGUMENT, "failed to create array");
@@ -147,18 +253,18 @@ tiny_array_init(grn_ctx *ctx, grn_array *array, const char *path,
 
 static grn_rc
 io_array_init(grn_ctx *ctx, grn_array *array, const char *path,
-               uint32_t value_size, uint32_t flags)
+              uint32_t value_size, uint32_t flags)
 {
   grn_io *io;
   struct grn_array_header *header;
   uint32_t w_of_element;
-  for (w_of_element = 0; (1 << w_of_element) < value_size; w_of_element++);
+  for (w_of_element = 0; (1U << w_of_element) < value_size; w_of_element++);
   {
     grn_io_array_spec array_spec[2];
     array_spec[array_seg_value].w_of_element = w_of_element;
-    array_spec[array_seg_value].max_n_segments = 1 << (30 - (22 - w_of_element));
+    array_spec[array_seg_value].max_n_segments = 1U << (30 - (22 - w_of_element));
     array_spec[array_seg_bitmap].w_of_element = 0;
-    array_spec[array_seg_bitmap].max_n_segments = 1 << (30 - (22 + 3));
+    array_spec[array_seg_bitmap].max_n_segments = 1U << (30 - (22 + 3));
     io = grn_io_create_with_array(ctx, path, sizeof(struct grn_array_header),
                                   GRN_ARRAY_SEGMENT_SIZE, grn_io_auto, 2, array_spec);
   }
@@ -189,9 +295,11 @@ inline static grn_array *
 _grn_array_create(grn_ctx *ctx, grn_array *array,
                   const char *path, uint32_t value_size, uint32_t flags)
 {
-  if (!((flags & GRN_ARRAY_TINY) ?
-        tiny_array_init(ctx, array, path, value_size, flags) :
-        io_array_init(ctx, array, path, value_size, flags))) {
+  if (flags & GRN_ARRAY_TINY) {
+    if (!tiny_array_init(ctx, array, path, value_size, flags)) {
+      return array;
+    }
+  } else if (!io_array_init(ctx, array, path, value_size, flags)) {
     return array;
   }
   return NULL;
@@ -311,10 +419,8 @@ grn_array_get_value(grn_ctx *ctx, grn_array *array, grn_id id, void *valuebuf)
 {
   if (ctx && array) {
     void *ee;
-    uint8_t res;
-    ARRAY_BITMAP_AT(array, id, res);
-    if (!res) { return 0; }
-    ARRAY_ENTRY_AT(array, id, ee, 0);
+    if (!grn_array_bitmap_at(ctx, array, id)) { return 0; }
+    ee = grn_array_entry_at(ctx, array, id, 0);
     if (ee) {
       if (valuebuf) { memcpy(valuebuf, ee, array->value_size); }
       return array->value_size;
@@ -328,10 +434,8 @@ _grn_array_get_value(grn_ctx *ctx, grn_array *array, grn_id id)
 {
   if (ctx && array) {
     void *ee;
-    uint8_t res;
-    ARRAY_BITMAP_AT(array, id, res);
-    if (!res) { return NULL; }
-    ARRAY_ENTRY_AT(array, id, ee, 0);
+    if (!grn_array_bitmap_at(ctx, array, id)) { return NULL; }
+    ee = grn_array_entry_at(ctx, array, id, 0);
     return ee;
   }
   return NULL;
@@ -343,10 +447,8 @@ grn_array_set_value(grn_ctx *ctx, grn_array *array, grn_id id,
 {
   if (ctx && array && value) {
     void *ee;
-    uint8_t res;
-    ARRAY_BITMAP_AT(array, id, res);
-    if (!res) { return GRN_INVALID_ARGUMENT; }
-    ARRAY_ENTRY_AT(array, id, ee, 0);
+    if (!grn_array_bitmap_at(ctx, array, id)) { return GRN_INVALID_ARGUMENT; }
+    ee = grn_array_entry_at(ctx, array, id, 0);
     if (ee) {
       switch ((flags & GRN_OBJ_SET_MASK)) {
       case GRN_OBJ_SET :
@@ -391,17 +493,15 @@ grn_rc
 grn_array_delete_by_id(grn_ctx *ctx, grn_array *array, grn_id id,
                        grn_table_delete_optarg *optarg)
 {
-  uint8_t res;
   grn_rc rc = GRN_SUCCESS;
   if (!ctx || !array) { return GRN_INVALID_ARGUMENT; }
-  ARRAY_BITMAP_AT(array, id, res);
-  if (!res) { rc = GRN_INVALID_ARGUMENT; goto exit; }
+  if (!grn_array_bitmap_at(ctx, array, id)) { rc = GRN_INVALID_ARGUMENT; goto exit; }
   /* lock */
   if (IO_ARRAYP(array)) {
     if (array->value_size >= sizeof(grn_id)) {
       void *ee;
       struct grn_array_header *hh = array->header;
-      ARRAY_ENTRY_AT_(array, id, ee, 0);
+      ee = grn_array_io_entry_at(ctx, array, id, 0);
       if (!ee) { rc = GRN_INVALID_ARGUMENT; goto exit; }
       *((grn_id *)ee) = hh->garbages;
       hh->garbages = id;
@@ -411,15 +511,14 @@ grn_array_delete_by_id(grn_ctx *ctx, grn_array *array, grn_id id,
     GRN_IO_ARRAY_BIT_OFF(array->io, array_seg_bitmap, id);
   } else {
     if (array->value_size >= sizeof(grn_id)) {
-      void *ee;
-      GRN_TINY_ARRAY_AT(&array->a, id, ee);
+      void *ee = grn_tiny_array_at_inline(&array->a, id);
       if (!ee) { rc = GRN_INVALID_ARGUMENT; goto exit; }
       *((grn_id *)ee) = array->garbages;
       array->garbages = id;
     }
     (*array->n_entries)--;
     (*array->n_garbages)++;
-    GRN_TINY_ARRAY_BIT_OFF(&array->bitmap, id);
+    grn_tiny_array_bit_off(&array->bitmap, id);
   }
 exit :
   /* unlock */
@@ -429,9 +528,7 @@ exit :
 grn_id
 grn_array_at(grn_ctx *ctx, grn_array *array, grn_id id)
 {
-  uint8_t res;
-  ARRAY_BITMAP_AT(array, id, res);
-  return res ? id : GRN_ID_NIL;
+  return grn_array_bitmap_at(ctx, array, id) ? id : GRN_ID_NIL;
 }
 
 grn_rc
@@ -451,8 +548,11 @@ grn_array_cursor_close(grn_ctx *ctx, grn_array_cursor *c)
   GRN_FREE(c);
 }
 
-#define ARRAY_CURR_MAX(array) \
-  ((IO_ARRAYP(array)) ? (array)->header->curr_rec : (array)->a.max)
+inline static grn_id
+grn_array_get_max_id(grn_array *array)
+{
+  return IO_ARRAYP(array) ? array->header->curr_rec : array->a.max;
+}
 
 grn_array_cursor *
 grn_array_cursor_open(grn_ctx *ctx, grn_array *array, grn_id min, grn_id max,
@@ -472,7 +572,7 @@ grn_array_cursor_open(grn_ctx *ctx, grn_array *array, grn_id min, grn_id max,
       c->curr_rec = max;
       if (!(flags & GRN_CURSOR_LT)) { c->curr_rec++; }
     } else {
-      c->curr_rec = ARRAY_CURR_MAX(array) + 1;
+      c->curr_rec = grn_array_get_max_id(array) + 1;
     }
     if (min) {
       c->tail = min;
@@ -493,16 +593,14 @@ grn_array_cursor_open(grn_ctx *ctx, grn_array *array, grn_id min, grn_id max,
       c->tail = max;
       if ((flags & GRN_CURSOR_LT)) { c->tail--; }
     } else {
-      c->tail = ARRAY_CURR_MAX(array);
+      c->tail = grn_array_get_max_id(array);
     }
     if (c->tail < c->curr_rec) { c->tail = c->curr_rec; }
   }
   if (*array->n_garbages) {
     while (offset && c->curr_rec != c->tail) {
-      uint8_t res;
       c->curr_rec += c->dir;
-      ARRAY_BITMAP_AT(c->array, c->curr_rec, res);
-      if (res) { offset--; }
+      if (grn_array_bitmap_at(ctx, c->array, c->curr_rec)) { offset--; }
     }
   } else {
     c->curr_rec += c->dir * offset;
@@ -518,9 +616,7 @@ grn_array_cursor_next(grn_ctx *ctx, grn_array_cursor *c)
     while (c->curr_rec != c->tail) {
       c->curr_rec += c->dir;
       if (*c->array->n_garbages) {
-        uint8_t res;
-        ARRAY_BITMAP_AT(c->array, c->curr_rec, res);
-        if (!res) { continue; }
+        if (!grn_array_bitmap_at(ctx, c->array, c->curr_rec)) { continue; }
       }
       c->rest--;
       return c->curr_rec;
@@ -532,11 +628,9 @@ grn_array_cursor_next(grn_ctx *ctx, grn_array_cursor *c)
 grn_id
 grn_array_next(grn_ctx *ctx, grn_array *array, grn_id id)
 {
-  grn_id max = ARRAY_CURR_MAX(array);
+  grn_id max = grn_array_get_max_id(array);
   while (++id <= max) {
-    uint8_t res;
-    ARRAY_BITMAP_AT(array, id, res);
-    if (res) { return id; }
+    if (grn_array_bitmap_at(ctx, array, id)) { return id; }
   }
   return GRN_ID_NIL;
 }
@@ -546,7 +640,7 @@ grn_array_cursor_get_value(grn_ctx *ctx, grn_array_cursor *c, void **value)
 {
   void *ee;
   if (c && value) {
-    ARRAY_ENTRY_AT(c->array, c->curr_rec, ee, 0);
+    ee = grn_array_entry_at(ctx, c->array, c->curr_rec, 0);
     if (ee) {
       *value = ee;
       return c->array->value_size;
@@ -576,8 +670,7 @@ array_entry_new(grn_ctx *ctx, grn_array *array)
   if (IO_ARRAYP(array)) {
     struct grn_array_header *hh = array->header;
     if ((e = hh->garbages)) {
-      void *ee;
-      ARRAY_ENTRY_AT_(array, e, ee, GRN_TABLE_ADD);
+      void *ee = grn_array_io_entry_at(ctx, array, e, GRN_TABLE_ADD);
       if (!ee) { return GRN_ID_NIL; }
       hh->garbages = *((grn_id *)ee);
       memset(ee, 0, hh->value_size);
@@ -588,15 +681,14 @@ array_entry_new(grn_ctx *ctx, grn_array *array)
     GRN_IO_ARRAY_BIT_ON(array->io, array_seg_bitmap, e);
   } else {
     if ((e = array->garbages)) {
-      void *ee;
-      GRN_TINY_ARRAY_AT(&array->a, e, ee);
+      void *ee = grn_tiny_array_at_inline(&array->a, e);
       array->garbages = *((grn_id *)ee);
       memset(ee, 0, array->value_size);
       (*array->n_garbages)--;
     } else {
       e = ++array->a.max;
     }
-    GRN_TINY_ARRAY_BIT_ON(&array->bitmap, e);
+    grn_tiny_array_bit_on(&array->bitmap, e);
   }
   return e;
 }
@@ -608,15 +700,15 @@ grn_array_add(grn_ctx *ctx, grn_array *array, void **value)
   void *ee;
   if (!ctx || !array || !(e = array_entry_new(ctx, array))) { return GRN_ID_NIL; }
   (*array->n_entries)++;
-  ARRAY_ENTRY_AT(array, e, ee, GRN_TABLE_ADD);
+  ee = grn_array_entry_at(ctx, array, e, GRN_TABLE_ADD);
   if (value) { *value = ee; }
   return e;
 }
 
 /* grn_hash : hash table */
 
-#define GRN_HASH_MAX_SEGMENT 0x400
-#define GRN_HASH_HEADER_SIZE 0x9000
+#define GRN_HASH_MAX_SEGMENT  0x400
+#define GRN_HASH_HEADER_SIZE  0x9000
 #define GRN_HASH_SEGMENT_SIZE 0x400000
 #define W_OF_KEY_IN_A_SEGMENT 22
 #define IDX_MASK_IN_A_SEGMENT 0xfffff
@@ -653,27 +745,27 @@ enum {
   segment_bitmap = 3
 };
 
-#define ENTRY_AT_(hash,id,ee,addp) {\
+#define ENTRY_AT_(hash,id,ee,addp) do {\
   int flags = addp;\
   GRN_IO_ARRAY_AT(hash->io, segment_entry, id, &flags, ee);\
-}
+} while (0)
 
 // todo : error handling
-#define ENTRY_AT(hash,id,ee,addp) {\
+#define ENTRY_AT(hash,id,ee,addp) do {\
   if (IO_HASHP(hash)) {\
     ENTRY_AT_(hash, id, ee, addp);\
   } else {\
-    GRN_TINY_ARRAY_AT(&hash->a, id, ee);\
+    (ee) = grn_tiny_array_at_inline(&hash->a, id);\
   }\
-}
+} while (0)
 
-#define BITMAP_AT(hash,id,value) {\
+#define BITMAP_AT(hash,id,value) do {\
   if (IO_HASHP(hash)) {\
     GRN_IO_ARRAY_BIT_AT((hash)->io, segment_bitmap, (id), (value));\
   } else {\
-    GRN_TINY_ARRAY_BIT_AT(&(hash)->bitmap, (id), (value));\
+    (value) = grn_tiny_array_bit_at(&(hash)->bitmap, (id));\
   }\
-}
+} while (0)
 
 inline static grn_id *
 idx_at_(grn_ctx *ctx, grn_hash *hash, grn_id id)
@@ -684,17 +776,15 @@ idx_at_(grn_ctx *ctx, grn_hash *hash, grn_id id)
   return pp;
 }
 
-#define IDX_AT(h,i) (\
-  IO_HASHP(h) ?\
-  idx_at_(ctx, h, ((i) & *(h)->max_offset) + h->header->idx_offset) :\
-  h->index + ((i) & *(h)->max_offset)\
-)
+#define IDX_AT(h,i) \
+  (IO_HASHP(h) ?\
+   idx_at_(ctx, h, ((i) & *(h)->max_offset) + h->header->idx_offset) :\
+   h->index + ((i) & *(h)->max_offset))
 
-#define KEY_AT(hash,pos,ptr) \
-{\
+#define KEY_AT(hash,pos,ptr) do {\
   int flags = GRN_TABLE_ADD;\
   GRN_IO_ARRAY_AT(hash->io, segment_key, pos, &flags, ptr);\
-}
+} while (0)
 
 #define HASH_IMMEDIATE 1
 
@@ -767,7 +857,8 @@ put_key_(grn_ctx *ctx, grn_hash *hash, entry_str *n, const char *key, int len)
 }
 
 inline static grn_rc
-put_key(grn_ctx *ctx, grn_hash *hash, entry_str *n, uint32_t h, const char *key, unsigned int len)
+put_key(grn_ctx *ctx, grn_hash *hash, entry_str *n, uint32_t h,
+        const char *key, unsigned int len)
 {
   n->key = h;
   if (hash->obj.header.flags & GRN_OBJ_KEY_VAR_SIZE) {
@@ -802,7 +893,8 @@ put_key(grn_ctx *ctx, grn_hash *hash, entry_str *n, uint32_t h, const char *key,
 }
 
 inline static int
-match_key(grn_ctx *ctx, grn_hash *hash, entry_str *ee, uint32_t h, const char *key, unsigned int len)
+match_key(grn_ctx *ctx, grn_hash *hash, entry_str *ee, uint32_t h,
+          const char *key, unsigned int len)
 {
   if (hash->obj.header.flags & GRN_OBJ_KEY_VAR_SIZE) {
     return (ee->key == h &&
@@ -821,8 +913,8 @@ match_key(grn_ctx *ctx, grn_hash *hash, entry_str *ee, uint32_t h, const char *k
 
 inline static grn_rc
 io_hash_init(grn_hash *ih, grn_ctx *ctx, const char *path, uint32_t key_size,
-               uint32_t value_size, uint32_t flags, grn_encoding encoding,
-               uint32_t init_size)
+             uint32_t value_size, uint32_t flags, grn_encoding encoding,
+             uint32_t init_size)
 {
   grn_io *io;
   struct grn_hash_header *header;
@@ -837,17 +929,17 @@ io_hash_init(grn_hash *ih, grn_ctx *ctx, const char *path, uint32_t key_size,
       entry_size = (intptr_t)(&((entry *)0)->dummy) + key_size + value_size;
     }
   }
-  for (w_of_element = 0; (1 << w_of_element) < entry_size; w_of_element++);
+  for (w_of_element = 0; (1U << w_of_element) < entry_size; w_of_element++);
   {
     grn_io_array_spec array_spec[4];
     array_spec[segment_key].w_of_element = 0;
     array_spec[segment_key].max_n_segments = 0x400;
     array_spec[segment_entry].w_of_element = w_of_element;
-    array_spec[segment_entry].max_n_segments = 1 << (30 - (22 - w_of_element));
+    array_spec[segment_entry].max_n_segments = 1U << (30 - (22 - w_of_element));
     array_spec[segment_index].w_of_element = 2;
-    array_spec[segment_index].max_n_segments = 1 << (30 - (22 - 2));
+    array_spec[segment_index].max_n_segments = 1U << (30 - (22 - 2));
     array_spec[segment_bitmap].w_of_element = 0;
-    array_spec[segment_bitmap].max_n_segments = 1 << (30 - (22 + 3));
+    array_spec[segment_bitmap].max_n_segments = 1U << (30 - (22 + 3));
     io = grn_io_create_with_array(ctx, path, GRN_HASH_HEADER_SIZE, GRN_HASH_SEGMENT_SIZE,
                                   grn_io_auto, 4, array_spec);
   }
@@ -1006,7 +1098,7 @@ tiny_hash_fin(grn_ctx *ctx, grn_hash *hash)
         entry_astr *n;
         e = *sp;
         if (!e || (e == GARBAGE)) { continue; }
-        GRN_TINY_ARRAY_AT(&hash->a, e, n);
+        n = grn_tiny_array_at_inline(&hash->a, e);
         GRN_ASSERT(n);
         i--;
         if (!n || (n->flag & HASH_IMMEDIATE)) { continue; }
@@ -1174,13 +1266,13 @@ entry_new(grn_ctx *ctx, grn_hash *hash, uint32_t size)
     if (hash->garbages) {
       entry *ee;
       e = hash->garbages;
-      GRN_TINY_ARRAY_AT(&hash->a, e, ee);
+      ee = grn_tiny_array_at_inline(&hash->a, e);
       hash->garbages = ee->key;
       memset(ee, 0, hash->entry_size);
     } else {
       e = hash->a.max + 1;
     }
-    GRN_TINY_ARRAY_BIT_ON(&hash->bitmap, e);
+    grn_tiny_array_bit_on(&hash->bitmap, e);
   }
   return e;
 }
@@ -1499,7 +1591,7 @@ grn_hash_set_value(grn_ctx *ctx, grn_hash *hash, grn_id id,
   return GRN_INVALID_ARGUMENT;
 }
 
-#define DELETE_IT {\
+#define DELETE_IT do {\
   *ep = GARBAGE;\
   if (IO_HASHP(hash)) {\
     uint32_t size = key_size - 1;\
@@ -1514,12 +1606,12 @@ grn_hash_set_value(grn_ctx *ctx, grn_hash *hash, grn_id id,
       grn_ctx *ctx = hash->ctx;\
       GRN_CTX_FREE(ctx, ((entry_astr *)ee)->str);\
     }\
-    GRN_TINY_ARRAY_BIT_OFF(&hash->bitmap, e);\
+    grn_tiny_array_bit_off(&hash->bitmap, e);\
   }\
   (*hash->n_entries)--;\
   (*hash->n_garbages)++;\
   rc = GRN_SUCCESS;\
-}
+} while (0)
 
 grn_rc
 grn_hash_delete_by_id(grn_ctx *ctx, grn_hash *hash, grn_id id,
@@ -1777,7 +1869,7 @@ grn_hash_cursor_delete(grn_ctx *ctx, grn_hash_cursor *c,
 
 /* sort */
 
-#define PREPARE_VAL(e,ep,es) {\
+#define PREPARE_VAL(e,ep,es) do {\
   if ((arg->flags & GRN_TABLE_SORT_BY_VALUE)) {\
     ep = ((const uint8_t *)(get_value(hash, (entry_str *)(e))));\
     es = hash->value_size;\
@@ -1788,7 +1880,7 @@ grn_hash_cursor_delete(grn_ctx *ctx, grn_hash_cursor *c,
   }\
   ep += arg->offset;\
   es -= arg->offset;\
-}
+} while (0)
 
 #define COMPARE_VAL_(ap,as,bp,bs)\
   (arg->compar\
@@ -1853,13 +1945,13 @@ swap(entry **a, entry **b)
   *b = c_;
 }
 
-#define SWAP(a,ap,as,b,bp,bs) {\
+#define SWAP(a,ap,as,b,bp,bs) do {\
   const uint8_t *cp_ = ap;\
   uint32_t cs_ = as;\
   ap = bp; bp = cp_;\
   as = bs; bs = cs_;\
   swap(a,b);\
-}
+} while (0)
 
 inline static entry **
 part(grn_ctx *ctx, entry **b, entry **e, grn_table_sort_optarg *arg, grn_hash *hash, int dir)
@@ -1935,14 +2027,14 @@ typedef struct {
   int32_t v;
 } val32;
 
-#define PREPARE_VAL32(id,e,ep) {\
+#define PREPARE_VAL32(id,e,ep) do {\
   (ep)->id = id;\
   (ep)->v = (arg->flags & GRN_TABLE_SORT_BY_ID)\
     ? (int32_t) id\
     : (*((int32_t *)((byte *)((arg->flags & GRN_TABLE_SORT_BY_VALUE)\
                               ? get_value(hash, (e))\
                               : get_key(ctx, hash, (e))) + arg->offset)));\
-}
+} while (0)
 
 #define COMPARE_VAL32_(ap,bp) \
   (arg->compar\
@@ -1995,11 +2087,11 @@ pack_val32(grn_ctx *ctx, grn_hash *hash, val32 *res, grn_table_sort_optarg *arg,
   return *hash->n_entries > 2 ? head : NULL;
 }
 
-#define SWAP_VAL32(ap,bp) {\
+#define SWAP_VAL32(ap,bp) do {\
   val32 cr_ = *ap;\
   *ap = *bp;\
   *bp = cr_;\
-}
+} while (0)
 
 inline static val32 *
 part_val32(grn_ctx *ctx,
@@ -2122,7 +2214,7 @@ grn_hash_sort(grn_ctx *ctx, grn_hash *hash,
       sort(ctx, hash, res, limit, optarg, dir);
     }
   } else {
-    grn_table_sort_optarg opt = {0, NULL, NULL, 0};
+    grn_table_sort_optarg opt = {0, NULL, NULL, NULL, 0};
     sort(ctx, hash, res, limit, &opt, 0);
   }
   {
@@ -2182,7 +2274,7 @@ static uint32_t default_flags = GRN_HASH_TINY;
 
 grn_rc
 grn_rhash_init(grn_ctx *ctx, grn_hash *hash, grn_rec_unit record_unit, int record_size,
-              grn_rec_unit subrec_unit, int subrec_size, unsigned int max_n_subrecs)
+               grn_rec_unit subrec_unit, int subrec_size, unsigned int max_n_subrecs)
 {
   grn_rc rc;
   record_size = grn_rec_unit_size(record_unit, record_size);
@@ -2365,7 +2457,7 @@ grn_rhash_group(grn_hash *s, int limit, grn_group_optarg *optarg)
 
 grn_rc
 grn_rhash_subrec_info(grn_hash *s, grn_id rh, int index,
-                     grn_id *rid, int *section, int *pos, int *score, void **subrec)
+                      grn_id *rid, int *section, int *pos, int *score, void **subrec)
 {
   grn_rset_posinfo *pi;
   grn_rset_recinfo *ri;
