@@ -36,7 +36,7 @@
 
 /* VA hack */
 /* max aio request (/proc/sys/fs/aio-max-nr) */
-#define MAX_REQUEST     (64*1024)
+#define MAX_REQUEST  (64*1024)
 
 #define MEM_ALIGN    grn_cache_block
 
@@ -381,7 +381,7 @@ grn_io_detect_type(grn_ctx *ctx, const char *path)
 {
   struct _grn_io_header h;
   uint32_t res = 0;
-  int fd = open(path, O_RDWR);
+  int fd = GRN_OPEN(path, O_RDWR);
   if (fd != -1) {
     struct stat s;
     if (fstat(fd, &s) != -1 && s.st_size >= sizeof(struct _grn_io_header)) {
@@ -397,7 +397,7 @@ grn_io_detect_type(grn_ctx *ctx, const char *path)
     } else {
       ERR(GRN_INVALID_FORMAT, "grn_io_detect_type failed");
     }
-    close(fd);
+    GRN_CLOSE(fd);
   } else {
     SERR(path);
   }
@@ -417,7 +417,7 @@ grn_io_open(grn_ctx *ctx, const char *path, grn_io_mode mode)
   if (!path || !*path || (strlen(path) > PATH_MAX - 4)) { return NULL; }
   {
     struct _grn_io_header h;
-    int fd = open(path, O_RDWR);
+    int fd = GRN_OPEN(path, O_RDWR);
     if (fd == -1) { SERR(path); return NULL; }
     if (fstat(fd, &s) != -1 && s.st_size >= sizeof(struct _grn_io_header)) {
       if (read(fd, &h, sizeof(struct _grn_io_header)) == sizeof(struct _grn_io_header)) {
@@ -431,7 +431,7 @@ grn_io_open(grn_ctx *ctx, const char *path, grn_io_mode mode)
         }
       }
     }
-    close(fd);
+    GRN_CLOSE(fd);
     if (!segment_size) { return NULL; }
   }
   total_header_size = IO_HEADER_SIZE + header_size;
@@ -1247,8 +1247,7 @@ grn_io_win_unmap2(grn_io_win *iw)
   }
 }
 
-#define DO_MAP(io,fmo,fi,pos,size,segno,res)\
-{\
+#define DO_MAP(io,fmo,fi,pos,size,segno,res) do {\
   if (((res) = GRN_MMAP(&grn_gctx, (fmo), (fi), (pos), (size)))) {\
     uint32_t nmaps;\
     if (io->max_map_seg < segno) { io->max_map_seg = segno; }\
@@ -1258,10 +1257,9 @@ grn_io_win_unmap2(grn_io_win *iw)
       if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
     }\
   }\
-}
+} while (0)
 
-#define SEG_MAP(io,segno,info)\
-{\
+#define SEG_MAP(io,segno,info) do {\
   uint32_t segment_size = io->header->segment_size;\
   if ((io->flags & GRN_IO_TEMPORARY)) {\
     DO_MAP(io, &info->fmo, NULL, 0, segment_size, segno, info->map);\
@@ -1282,7 +1280,7 @@ grn_io_win_unmap2(grn_io_win *iw)
       DO_MAP(io, &info->fmo, fi, pos, segment_size, segno, info->map);\
     }\
   }\
-}
+} while (0)
 
 void
 grn_io_seg_map_(grn_ctx *ctx, grn_io *io, uint32_t segno, grn_io_mapinfo *info)
@@ -1810,7 +1808,7 @@ inline static grn_rc
 grn_open(grn_ctx *ctx, fileinfo *fi, const char *path, int flags, size_t maxsize)
 {
   struct stat st;
-  if ((fi->fd = open(path, flags, 0666)) == -1) {
+  if ((fi->fd = GRN_OPEN(path, flags, 0666)) == -1) {
     SERR(path);
     return ctx->rc;
   }
@@ -1839,7 +1837,7 @@ inline static grn_rc
 grn_close(grn_ctx *ctx, fileinfo *fi)
 {
   if (fi->fd != -1) {
-    if (close(fi->fd) == -1) {
+    if (GRN_CLOSE(fi->fd) == -1) {
       SERR("close");
       return ctx->rc;
     }
