@@ -45,7 +45,7 @@ grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
   /* FIXME: implement more smartly with grn_bulk */
   int fd, ret = 0;
   struct stat stat;
-  if ((fd = open(path, O_RDONLY|O_NOFOLLOW)) == -1) {
+  if ((fd = GRN_OPEN(path, O_RDONLY|O_NOFOLLOW)) == -1) {
     switch (errno) {
     case EACCES :
       ERR(GRN_OPERATION_NOT_PERMITTED, "request is not allowed: <%s>", path);
@@ -60,7 +60,7 @@ grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
       break;
 #endif /* WIN32 */
     default :
-      ERR(GRN_UNKNOWN_ERROR, "open() failed(errno: %d): <%s>", errno, path);
+      ERR(GRN_UNKNOWN_ERROR, "GRN_OPEN() failed(errno: %d): <%s>", errno, path);
       break;
     }
     return 0;
@@ -71,7 +71,7 @@ grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
     if ((buf = GRN_MALLOC(rest))) {
       ssize_t ss;
       for (bp = buf; rest; rest -= ss, bp += ss) {
-        if ((ss = read(fd, bp, rest)) == -1) { goto exit; }
+        if ((ss = GRN_READ(fd, bp, rest)) == -1) { goto exit; }
       }
       GRN_TEXT_PUT(ctx, bulk, buf, stat.st_size);
       ret = 1;
@@ -81,7 +81,7 @@ grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
     ERR(GRN_INVALID_ARGUMENT, "cannot stat file: <%s>", path);
   }
 exit :
-  close(fd);
+  GRN_CLOSE(fd);
   return ret;
 }
 
@@ -1984,35 +1984,6 @@ dump_column(grn_ctx *ctx, grn_obj *outbuf , grn_obj *table, grn_obj *column)
   grn_obj_unlink(ctx, type);
 }
 
-static int
-have_index_column(grn_ctx *ctx, grn_obj *table)
-{
-  int n_index_columns = 0;
-  grn_hash *columns;
-  columns = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
-                            GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY);
-  if (!columns) {
-    ERR(GRN_ERROR, "couldn't create a hash to hold columns");
-    return 0;
-  }
-
-  if (grn_table_columns(ctx, table, NULL, 0, (grn_obj *)columns) >= 0) {
-    grn_id *key;
-
-    GRN_HASH_EACH(ctx, columns, id, &key, NULL, NULL, {
-      grn_obj *column;
-      if ((column = grn_ctx_at(ctx, *key))) {
-        if (column->header.flags & GRN_OBJ_COLUMN_INDEX) {
-          n_index_columns++;
-        }
-        grn_obj_unlink(ctx, column);
-      }
-    });
-  }
-  grn_hash_close(ctx, columns);
-  return n_index_columns;
-}
-
 static int
 reference_column_p(grn_ctx *ctx, grn_obj *column)
 {
@@ -2110,9 +2081,7 @@ dump_records(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table)
     return;
   }
 
-  if (grn_table_size(ctx, table) == 0 ||
-      (grn_obj_get_info(ctx, table, GRN_INFO_DEFAULT_TOKENIZER, NULL) &&
-       have_index_column(ctx, table))) {
+  if (grn_table_size(ctx, table) == 0) {
     return;
   }
 
@@ -2331,14 +2300,14 @@ dump_table(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table,
 }
 
 /* can we move this to groonga.h? */
-#define GRN_PTR_POP(obj,value) {\
+#define GRN_PTR_POP(obj,value) do {\
   if (GRN_BULK_VSIZE(obj) >= sizeof(grn_obj *)) {\
     GRN_BULK_INCR_LEN((obj), -(sizeof(grn_obj *)));\
     value = *(grn_obj **)(GRN_BULK_CURR(obj));\
   } else {\
     value = NULL;\
   }\
-}
+} while (0)
 
 static void
 dump_scheme(grn_ctx *ctx, grn_obj *outbuf)
@@ -2827,11 +2796,11 @@ func_edit_distance(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_
   return obj;
 }
 
-#define DEF_VAR(v,name_str) {\
+#define DEF_VAR(v,name_str) do {\
   (v).name = (name_str);\
   (v).name_size = GRN_STRLEN(name_str);\
   GRN_TEXT_INIT(&(v).value, 0);\
-}
+} while (0)
 
 #define DEF_COMMAND(name, func, nvars, vars)\
   (grn_proc_create(ctx, (name), (sizeof(name) - 1),\
