@@ -245,28 +245,28 @@ struct _grn_ja_einfo {
 #define ETINY_ENC(e,_size) ((e)->u.c[7] = (_size) + ETINY)
 #define ETINY_DEC(e,_size) ((_size) = (e)->u.c[7] & ~(ETINY|EHUGE))
 #define EHUGE_P(e) ((e)->u.c[7] & EHUGE)
-#define EHUGE_ENC(e,_seg,_size) {\
+#define EHUGE_ENC(e,_seg,_size) do {\
   (e)->u.h.c1 = 0;\
   (e)->u.h.c2 = EHUGE;\
   (e)->u.h.seg = (_seg);\
   (e)->u.h.size = (_size);\
-}
-#define EHUGE_DEC(e,_seg,_size) {\
+} while (0)
+#define EHUGE_DEC(e,_seg,_size) do {\
   (_seg) = (e)->u.h.seg;\
   (_size) = (e)->u.h.size;\
-}
-#define EINFO_ENC(e,_seg,_pos,_size) {\
+} while (0)
+#define EINFO_ENC(e,_seg,_pos,_size) do {\
   (e)->u.n.c1 = (_pos) >> 16;\
   (e)->u.n.c2 = ((_size) >> 16);\
   (e)->u.n.seg = (_seg);\
   (e)->u.n.pos = (_pos);\
   (e)->u.n.size = (_size);\
-}
-#define EINFO_DEC(e,_seg,_pos,_size) {\
+} while (0)
+#define EINFO_DEC(e,_seg,_pos,_size) do {\
   (_seg) = (e)->u.n.seg;\
   (_pos) = ((e)->u.n.c1 << 16) + (e)->u.n.pos;\
   (_size) = ((e)->u.n.c2 << 16) + (e)->u.n.size;\
-}
+} while (0)
 
 typedef struct {
   uint32_t seg;
@@ -564,17 +564,15 @@ grn_ja_replace(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_ja_einfo *ei, uint64_t *
   pseg = &ja->header->esegs[lseg];
   if (grn_io_lock(ctx, ja->io, 10000000)) { return ctx->rc; }
   if (*pseg == JA_ESEG_VOID) {
-    if (*pseg == JA_ESEG_VOID) {
-      int i = 0;
-      while (SEGMENTS_AT(ja, i)) {
-        if (++i >= JA_N_DSEGMENTS) {
-          ERR(GRN_NOT_ENOUGH_SPACE, "grn_ja file (%s) is full", ja->io->path);
-          goto exit;
-        }
+    int i = 0;
+    while (SEGMENTS_AT(ja, i)) {
+      if (++i >= JA_N_DSEGMENTS) {
+        ERR(GRN_NOT_ENOUGH_SPACE, "grn_ja file (%s) is full", ja->io->path);
+        goto exit;
       }
-      SEGMENTS_EINFO_ON(ja, i, lseg);
-      *pseg = i;
     }
+    SEGMENTS_EINFO_ON(ja, i, lseg);
+    *pseg = i;
   }
   GRN_IO_SEG_REF(ja->io, *pseg, einfo);
   if (!einfo) { goto exit; }
@@ -585,7 +583,11 @@ grn_ja_replace(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_ja_einfo *ei, uint64_t *
     goto exit;
   }
   // smb_wmb();
-  GRN_SET_64BIT(&einfo[pos], *ei);
+  {
+    uint64_t *location = (uint64_t *)(einfo + pos);
+    uint64_t value = *((uint64_t *)ei);
+    GRN_SET_64BIT(location, value);
+  }
   GRN_IO_SEG_UNREF(ja->io, *pseg);
   grn_ja_free(ctx, ja, &eback);
 exit :
@@ -1161,8 +1163,8 @@ grn_ja_get_value(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_obj *value)
       uint32_t el = len - sizeof(uint32_t);
       uint32_t pos = *((uint32_t *)(b + el));
       GRN_ASSERT(pos < el);
-      grn_bulk_write(ctx, value, b + pos, el - pos);
-      grn_bulk_write(ctx, value, b, pos);
+      grn_bulk_write(ctx, value, (char *)(b + pos), el - pos);
+      grn_bulk_write(ctx, value, (char *)(b), pos);
     } else {
       grn_bulk_write(ctx, value, v, len);
     }
