@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "lib/com.h"
@@ -285,7 +285,9 @@ do_client()
           if (!(nsent % 1000)) { lprint(ctx, "     : %d", nsent); }
         }
         done = 1;
-        THREAD_JOIN(thread);
+        if (THREAD_JOIN(thread)) {
+          fprintf(stderr, "THREAD_JOIN failed\n");
+        }
         gettimeofday(&tve, NULL);
         {
           double qps;
