@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <gcutter.h>
@@ -495,8 +495,8 @@ test_invalid_table_name(void)
   grn_test_assert_send_command_error(
     context,
     GRN_INVALID_ARGUMENT,
-    "[table][load]: name can't start with '_' "
-    "and contains only 0-9, A-Z, a-z, #, - or _: <_Users>",
+    "[table][load] name can't start with '_' "
+    "and contains only 0-9, A-Z, a-z, #, @, - or _: <_Users>",
     "load --table _Users\n"
     "[\n"
     "{\"_key\": \"alice\"}\n"
