@@ -14,17 +14,17 @@ if test x"$NO_MAKE" != x"yes"; then
 fi
 
 if test -z "$CUTTER"; then
-    CUTTER="`make -s -C $BUILD_DIR echo-cutter`"
+    CUTTER="`make -s -C $top_dir echo-cutter`"
 fi
 export CUTTER
 
 if test -z "$RUBY"; then
-    RUBY="`make -s -C $BUILD_DIR echo-ruby`"
+    RUBY="`make -s -C $top_dir echo-ruby`"
 fi
 export RUBY
 
 if test -z "$GROONGA"; then
-    GROONGA="`make -s -C $BUILD_DIR echo-groonga`"
+    GROONGA="`make -s -C $top_dir echo-groonga`"
 fi
 export GROONGA
 
@@ -80,8 +80,11 @@ ruby_result=0
 if test "$NO_RUBY" != "yes" -a -n "$RUBY"; then
     : ${TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE:=30000}
     export TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE
-    BUNDLE_GEMFILE=$BASE_DIR/Gemfile
+    BUNDLE_GEMFILE="$BUILD_DIR/Gemfile"
     export BUNDLE_GEMFILE
+    if [ ! -e "$BUNDLE_GEMFILE" ]; then
+	ln -s "$BASE_DIR/Gemfile" "$BUNDLE_GEMFILE"
+    fi
     if ! type bundle > /dev/null; then
 	$RUBY -S gem install bundler
     fi
